% *****************************************************************
% authors:  Fei Chen, Gene Cheung, Xue Zhang                     **
% paper:    Manifold Graph Signal Restoration using Gradient     **
%           Graph Laplacian Regularizer, submitted to TSP, 2022  **
% date:     12/3/2022                                            **
% modified: 12/3/2022                                            **
% *****************************************************************

clear all;
close all;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
x=ones(40,40);
[sx,sy]=size(x);
x=zeros(sx,sy);
for i=1:20
    x(i,i:20)=1:(21-i);
    x(i,21:(40-i+1))=(21-i):-1:1;
    x(i:20,i)=1:(21-i);
    x(21:(40-i+1),i)=(21-i):-1:1;
end
for i=1:40
    for j=1:i
        x(i,40-j+1)=x(j,i);
    end
end
x=x./20;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
sigma_noise=10/255;
randn('seed',0);
y = x+sigma_noise*randn(size(x));
[w, p] = Img2adj_mat(y);
mask=ones(size(y));

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
par.mu = 0.1;
par.nsig=10;
par.IterNum=7;

tic
[z, alpha1,alpha2] = GGLR(p,y(:),w,mask(:),par);
toc

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
disp(['GGLR:' num2str(psnr(x(:),y(:))) 'dB  --> ' num2str(psnr(x(:),z(:))) 'dB.']);
subplot(131)
imshow(reshape(x,sx,sy));
title('Original Patch');
subplot(132)
imshow(reshape(y,sx,sy));
title(['Corrupted:' num2str(psnr(x(:),y(:))) 'dB']);
subplot(133)
imshow(reshape(z,sx,sy));
title(['GGLR:' num2str(psnr(x(:),z(:))) 'dB']);
