% *****************************************************************
% authors:  Fei Chen, Gene Cheung, Xue Zhang                     **
% paper:    Manifold Graph Signal Restoration using Gradient     **
%           Graph Laplacian Regularizer, submitted to TSP, 2022  **
% date:     12/3/2022                                            **
% modified: 12/3/2022                                            **
% *****************************************************************

clear all;
close all;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
x=zeros(40,40);
[sx,sy]=size(x);
for i=1:40
    for j=1:40
        x(i,j)=sin(0.8*(2*i+j)/pi)+1;
    end
end
x(x<0)=0;
x(x>2)=2;
x=x./2;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
sigma_noise=20/255;
randn('seed',0);
y = x+sigma_noise*randn(size(x));
[w, p] = Img2adj_mat(y);
mask=ones(size(y));

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
par.mu = .07;
par.nsig=20;
par.IterNum=3;

tic
[z, alpha1,alpha2] = GGLR(p,y(:),w,mask(:),par);
toc

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
disp(['GGLR:' num2str(psnr(x(:),y(:))) 'dB  --> ' num2str(psnr(x(:),z(:))) 'dB.']);
subplot(131)
imshow(reshape(x,sx,sy));
title('Original Patch');
subplot(132)
imshow(reshape(y,sx,sy));
title(['Corrupted:' num2str(psnr(x(:),y(:))) 'dB']);
subplot(133)
imshow(reshape(z,sx,sy));
title(['GGLR:' num2str(psnr(x(:),z(:))) 'dB']);
