% *****************************************************************
% authors:  Fei Chen, Gene Cheung, Xue Zhang                     **
% paper:    Manifold Graph Signal Restoration using Gradient     **
%           Graph Laplacian Regularizer, submitted to TSP, 2022  **
% date:     12/3/2022                                            **
% modified: 12/3/2022                                            **
% *****************************************************************

function [z,alpha_1,alpha_2] = GGLR(p,y,w,mask,par )
N = size(w,1);
nsig = par.nsig;
mu = par.mu;
Loc = zeros(N,4);
len_Loc=zeros(N,1);

F_1 = sparse(N,N);
F_2 = sparse(N,N);
CF_1 = sparse(N,N);
CF_2 = sparse(N,N);

idset=1:N;
for i = 1:N
    id=idset(w(i,1:end)>0);
    len=length(id);
    Loc(i,1:len) = id;
    len_Loc(i,1)=len;
    
    a =1;
    for j = 1:len
        if  (p(1,id(j)) > p(1,i))
            F_1(i,id(j)) = -a;
            F_1(i,i) = F_1(i,i)+a;
            pp1=[p(1,i)-p(1,id(j)) p(2,i)-p(2,id(j))];
        end
        
        if (p(2,id(j))>p(2,i))
            F_2(i,id(j)) = -a;
            F_2(i,i) = F_2(i,i)+a;
            pp2=[p(1,i)-p(1,id(j)) p(2,i)-p(2,id(j))];
        end
    end
    
    Ci=[pp1;pp2];
    INVC=pinv(Ci);
    Fi=[F_1(i,:);F_2(i,:)];
    CFi=INVC*Fi;
    
    CF_1(i,:)=CFi(1,:);
    CF_2(i,:)=CFi(2,:);
end

z = y;

H = sparse(1:N,1:N,mask(:),N,N)+speye(N).*eps;

for step=1:par.IterNum
    
    alpha_1 = CF_1*z(:);
    alpha_2 = CF_2*z(:);
    
    nsig1 = nsig*(mean((abs(alpha_1(1:end-1)-alpha_1(2:end))))).^2;
    nsig2 = nsig*(mean((abs(alpha_2(1:end-1)-alpha_2(2:end))))).^2;
    
    W = sparse(N,N);
    
    for i = 1:N
        b = Loc(i,1:len_Loc(i,1));
        dis_alpha=(alpha_1(b,1)-alpha_1(i,1)).^2./nsig1+(alpha_2(b,1)-alpha_2(i,1)).^2./nsig2;
        W(b,i) = exp(-dis_alpha);
    end
    
    W = W+W';
    
    DW = diag(sum(W,2));
    L = DW - W;
    
    cL = CF_1'*L*CF_1+CF_2'*L*CF_2;
    
    if min(mask(:))==1
        z = (H + mu*(cL))\z(:);
    else
        z = (H + mu*(cL))\y(:);
    end
    z(z<0) = 0;
    z(z>1) = 1;
    
end
end

